#!/usr/bin/env perl
##!/usr/local/bin/perl -w
#
#
# remediation_1xEVDO.pl
#
#         This script is used to implement data remediation for Nortel 1xEVDO
#
# Usage:  remediation_1xEVDO.pl <config filename> <DC filename>
#
#
# Example:
#         remediation_1xEVDO.pl NortelEVDO.cfg MTX41.1.20031029070000.1xEVDO

use lib "$ENV{ FLEXPM_HOME }/vendor/Nortel/MSC/1.3/1xEVDO/scripts/";
use Tool;

#Global Variables
$MAX_VALUE=2**32 - 500000000;
$TWO_POWER_32=2**32 ;
%basic_references = ();
%raw_hash = ();

usage("Invalid number of arguments") if ($#ARGV != 1);
$inFilePath = $ARGV[0];
$mtxRel = $ARGV[1];
$current_filename = `basename $inFilePath`;

$outputFile = substr ($inFilePath, 0, length($inFilePath)-3);
$outputFileDir = "\.\.\/\.\.\/\.\.\/ftpIN\/$mtxRel\/1xParsed\/";
$currFileDir = "\.\.\/\.\.\/\.\.\/ftpIN\/$mtxRel\/1xParsed\/";

#object Tool.pm
$tool = new Tool;

($second, $minute, $hour) = localtime();
$theTime = "$hour:$minute:$second ";

#open (CDF,$inFilePath) or die "Cannot open $currDcFile:$!\n";
# open $outputFile for writing output
terminalError("Could not redirect stdout to $outputFile") if (!open(outputFileHandle, "> $outputFileDir$outputFile"));

($temp1,$current_time_stamp,$temp2) = ($current_filename =~ /(\w+).(\w+).(.*)/);
$previous_file_name = get_previous_filename($temp1,$current_time_stamp,$temp2);

form_basic_reference($current_time_stamp);

#use Data::Dumper;
for $file_name ( $previous_file_name, $inFilePath ) {
	open ( CDF , "$currFileDir/$file_name" ) || print outputFileHandle "#Can't open file $currFileDir/$file_name :$!:\n";

	
	while ( $line = <CDF> ) {
		chomp($line);
		next if ($line =~ /^#/);
		next if ($line =~ /^PREV/);
		($time_reference,$actual_data) = split('&&',$line);
        $rem_type = substr($time_reference,4,1);
		if ( $rem_type ne 'R' ) {
			$time_reference = substr($time_reference,0,4).'N'.substr($time_reference,-2,2);
		}
#print Dumper [ %basic_references];

		$interval = substr($time_reference,5,2);
#		if ( grep(/^$time_reference$/,keys %basic_references) ) {
		if ( exists $basic_references{$time_reference} )  {
			$section = $basic_references{$time_reference};
			@origRec   = split(/\|/, $actual_data);
			$CDFEntity = $origRec[0];
			$PegValue  = $origRec[$#origRec ];
			$PegName   = $origRec[$#origRec - 1];
			$stTime    = $origRec[$#origRec - 2];
			@InstID    = @origRec[1 .. $#origRec - 3 ];
			$InstanceKey=join ("-",$PegName, @InstID);
			$main_hash{$CDFEntity}{$rem_type}{$InstanceKey}{$section} = [ $PegValue,$stTime,substr($time_reference,5,2) ];
			$main_hash{$CDFEntity}{$rem_type}{$InstanceKey}{'30mins'} = 'true' if $interval eq '30';
			$main_hash{$CDFEntity}{$rem_type}{$InstanceKey}{'15mins'} = 'true' if $interval eq '15';
			$main_hash{$CDFEntity}{$rem_type}{'allnegativeflag1'} = 0;
			$main_hash{$CDFEntity}{$rem_type}{'allnegativeflag2'} = 0;
		}
	}
}

#checks for no data in any of the sections
if ( not defined %main_hash ) {
	print outputFileHandle "# no enough info for data processing.  please check the data file!!\n";
	exit(1);
}

foreach $val (keys %main_hash) { #Entity
	foreach $type ( keys %{$main_hash{$val}} ) { #rem_type
	    $total_R_count = keys %{$main_hash{$val}{'R'}};
		#to avoid allnegativeflags keys
		$total_R_count = 0;
		map { $total_R_count++ if $_ !~ /allnegative/ } keys %{$main_hash{$val}{'R'}};
		#print "Rem_count :$total_R_count:\n";
		$diff_negative_count_2 = 0; $diff_negative_count_1 = 0 if $type eq 'R';
		foreach $pegs ( keys %{$main_hash{$val}{$type}} ) {
			$diffT1 = '';$diffT2 ='';
			$T1val        = defined ${$main_hash{$val}{$type}{$pegs}{'Section1'}}[0] ? ${$main_hash{$val}{$type}{$pegs}{'Section1'}}[0] : undef;
			$T2val        = defined ${$main_hash{$val}{$type}{$pegs}{'Section2'}}[0] ? ${$main_hash{$val}{$type}{$pegs}{'Section2'}}[0] : undef;;
			$T3val        = defined ${$main_hash{$val}{$type}{$pegs}{'Section3'}}[0] ? ${$main_hash{$val}{$type}{$pegs}{'Section3'}}[0] : undef;

			if ( $type eq 'R' ) {
			
				$diffT1 = $T2val - $T1val if (defined $T2val && defined $T1val);
				$diffT2 = $T3val - $T2val if (defined $T3val && defined $T2val);

				if ($diffT1 < 0) {
				   $diff_negative_count_1++;
				   #Rollover scenario
				   if ($T1val > $MAX_VALUE ) {
					  $diffT1 = $TWO_POWER_32 - $T1val + $T2val;
              # When T2val -T1 is greater than 4 billion (2 power 32) then that means we have 64 bit counter.
# We determined that this is a reset scenario not rollover due to the sheer value of 2 ** 64
                                     if ($diffT1 < 0) {
#                                        $diffT1 = $T2val;
                                        $diffT1 = 0;
                                     }


				   } else { # Reset scenario
#					  $diffT1 = $T2val;
					  $diffT1 = 0;
				   }
				}

				if ($diffT2 < 0) {
				   $diff_negative_count_2++;
				   if ($T2val > $MAX_VALUE ) {
					  $diffT2 = $TWO_POWER_32 - $T2val + $T3val;
                                          if ($diffT2 < 0) {
#                                               $diffT2 = $T3val;
                                               $diffT2 = 0;
                                           }

				   } else {
#					  $diffT2 = $T3val;
					  $diffT2 = 0;
				   }
				}

				##
				$main_hash{$val}{$type}{$pegs}{'diff1'} = defined $diffT1 ? $diffT1 : '';
				$main_hash{$val}{$type}{$pegs}{'diff2'} = defined $diffT2 ? $diffT2 : '';
			} else {
				$main_hash{$val}{$type}{$pegs}{'diff1'} = defined $T1val ? $T1val : '';
				$main_hash{$val}{$type}{$pegs}{'diff2'} = defined $T2val ? $T2val : '';
			}
		}

		#print "total_R_count :$total_R_count - diff_negative_count_2 :$diff_negative_count_2\n";
		$main_hash{$val}{$type}{'allnegativeflag1'} = 1 if ($total_R_count == $diff_negative_count_1 );
		$main_hash{$val}{$type}{'allnegativeflag2'} = 1 if ($total_R_count == $diff_negative_count_2 );
	}
}

#print Dumper [ %main_hash ];
#Final String Formation
$Section1_no_data  = 0; $Section2_no_data = 0;
=Start
	Section1 and Section2 Should not be Empty to find out Remediation Value. The pegtype should be 'R'
	If it is a Non-Remediation Peg and the interval is not 30, then prints all the
	Non-remediation pegs as it is
	Because for 30 minutes interval, nothing has to be printed if section1 or section2 should be empty
	When all the peg values are in negative the warning message should get printed.
	If any of the section is missing the appropriate warning message should get printed
=cut
foreach $val (keys %main_hash) { #Entity
    $section1_info = 0; $section2_info = 0; $section3_info = 0;
	foreach $type ( keys %{$main_hash{$val}} ) { #rem_type
		foreach $pegs ( keys %{$main_hash{$val}{$type}} ) {
			next if ( $pegs =~ /allnegativeflag/ );
			@returnKeyT2 = split (/-/, $pegs);
			$pegName     = shift @returnKeyT2;

			print outputFileHandle join ("\|",$val,@returnKeyT2,${$main_hash{$val}{$type}{$pegs}{'Section1'}}[1],$pegName,$main_hash{$val}{$type}{$pegs}{'diff1'}) , "\n" if ( $main_hash{$val}{$type}{$pegs}{'diff1'} ne '' && defined ${$main_hash{$val}{$type}{$pegs}{'Section1'}}[1] );
			$section1_info = 1 if ( not defined ${$main_hash{$val}{$type}{$pegs}{'Section1'}}[0] );

			#This is specific to Handling 30 mins Interval. when there is no data in section2
			$section2_info = 1 if ( (not defined ${$main_hash{$val}{$type}{$pegs}{'Section2'}}[0]) && ($main_hash{$val}{$type}{$pegs}{'30mins'} eq 'true') );
			#print the Second Section if need
			if ( $main_hash{$val}{$type}{$pegs}{'15mins'} eq 'true' ) {
				print outputFileHandle join ("\|",$val,@returnKeyT2,${$main_hash{$val}{$type}{$pegs}{'Section2'}}[1],$pegName,$main_hash{$val}{$type}{$pegs}{'diff2'}) , "\n" if ( $main_hash{$val}{$type}{$pegs}{'diff2'} ne '' && ${$main_hash{$val}{$type}{$pegs}{'Section2'}}[1] );
				$section2_info = 1 if ( not defined ${$main_hash{$val}{$type}{$pegs}{'Section2'}}[0] );
				$section3_info = 1 if ( (not defined ${$main_hash{$val}{$type}{$pegs}{'Section3'}}[0]) && ($main_hash{$val}{$type}{$pegs}{'diff2'} eq '') && ($type eq 'R') );
			}
		}
		
		if ( $type eq 'R' && $main_hash{$val}{$type}{'allnegativeflag1'} == 1 ) {
			print outputFileHandle "# Warning: No remediations for First section of $val.  System might be restarted ?\n";
		}
		if ( $type eq 'R' && $main_hash{$val}{$type}{'allnegativeflag2'} == 1 ) {
			print outputFileHandle "# Warning: No remediations for Second section of $val.  System might be restarted ?\n";
		}
	}
	print outputFileHandle "# Not Enough information in previous Interval - Section1\n" if ( $section1_info == 1 );
	print outputFileHandle "# Not Enough information in current  Interval - Section2\n" if ( $section2_info == 1 );
	print outputFileHandle "# Not Enough information in next     Interval - Section3\n" if ( $section3_info == 1 );
}


if ( $Section1_no_data == 1 ) {
	print outputFileHandle "# no enough info for data processing on section1.  please check the data file!!\n";
}
if ( $Section2_no_data == 1 ) {
	print outputFileHandle "# no enough info for data processing on section2.  please check the data file!!\n";
}
#print Dumper [ %main_hash ];

#####################
# This function forms the Basic references, which is created using the Timestamp
# for which the process goes
#              sub form_basic_reference 
#                          input : timestamp
#####################
sub form_basic_reference {
  my $dateStr = shift;
  $Current0 = $dateStr;
  $Prev1    = $tool->create_reference( $dateStr  , 'sub',15);
  $Prev2    = $tool->create_reference( $Prev1    , 'sub',15);
  $Current1 = $tool->create_reference( $dateStr  , 'add',15);
  $Current2 = $tool->create_reference( $Current1 , 'add',15);
  $prev1    = substr($Prev1,8,4);
  $prev2    = substr($Prev2,8,4);
  $current0 = substr($Current0,8,4);
  $current1 = substr($Current1,8,4);
  $basic_references{$prev1.   'N'.'15'} = 'Section1';
  $basic_references{$prev1.   'R'.'15'} = 'Section1';

  $basic_references{$prev2.   'N'.'30'} = 'Section1';
  $basic_references{$prev2.   'R'.'30'} = 'Section1';

  $basic_references{$current0.'N'.'15'} = 'Section2';
  $basic_references{$current0.'R'.'15'} = 'Section2';

  $basic_references{$current0.'N'.'30'} = 'Section2';
  $basic_references{$current0.'R'.'30'} = 'Section2';

  $basic_references{$current1.'N'.'15'} = 'Section3';
  $basic_references{$current1.'R'.'15'} = 'Section3';
}

#####################
# This function returns the previous filename 
# by getting the current filename details
#       sub get_previous_filename 
#                           input : <first_part_of_file_name> <time_stamp> <third_part_of_file_name>
#####################
sub get_previous_filename {
	$temp1      = shift;
	$time_stamp = shift;
	$temp2      = shift;
	
	return $temp1.'.'.$tool->create_reference($tool->create_reference($time_stamp,'sub',15),'sub',15).'.'.$temp2;
}

sub terminalError {
  print  STDERR "\nError. ", $_[0] , ".\n\n";
  exit(1);
}

sub usage {
  my $message = $_[0];
  my $progName = $0;

  $progName = `basename $0`;
  chomp $progName;

  print  STDERR "\nError: $message\n\n";
  print  STDERR "Usage: $progName <DC filename> \n\n";
  print  STDERR "Arguments:\n";
  print  STDERR "  <DC filename>         Current data collection filename\n";
  exit(1);
}
